@@ -80,7 +80,11 @@ static void stream_free(ALLEGRO_SAMPLE_INSTANCE *spl)
          for (i = _al_vector_size(&mixer->streams) - 1; i >= 0; i--) {
             ALLEGRO_SAMPLE_INSTANCE **slot = _al_vector_ref(&mixer->streams, i);
             ALLEGRO_SAMPLE_INSTANCE *spl = *slot;
+
             spl->parent.u.ptr = NULL;
+            spl->spl_read = NULL;
+            al_free(spl->matrix);
+            spl->matrix = NULL;
          }
 
          _al_vector_free(&mixer->streams);
@@ -116,27 +120,24 @@ void _al_kcm_detach_from_parent(ALLEGRO_SAMPLE_INSTANCE *spl)
       al_detach_voice(spl->parent.u.voice);
       return;
    }
-   
+
    mixer = spl->parent.u.mixer;
 
    /* Search through the streams and check for this one */
+   maybe_lock_mutex(mixer->ss.mutex);
    for (i = _al_vector_size(&mixer->streams) - 1; i >= 0; i--) {
       ALLEGRO_SAMPLE_INSTANCE **slot = _al_vector_ref(&mixer->streams, i);
 
       if (*slot == spl) {
-         maybe_lock_mutex(mixer->ss.mutex);
-
          _al_vector_delete_at(&mixer->streams, i);
          spl->parent.u.mixer = NULL;
          _al_kcm_stream_set_mutex(spl, NULL);
 
          spl->spl_read = NULL;
-
-         maybe_unlock_mutex(mixer->ss.mutex);
-
          break;
       }
    }
+   maybe_unlock_mutex(mixer->ss.mutex);
 
    al_free(spl->matrix);
    spl->matrix = NULL;
