@@ -24,7 +24,7 @@
 #endif
 
 /* Allow up to four extra axes for future expansion. */
-#define ALLEGRO_MOUSE_MAX_EXTRA_AXES	 4
+#define ALLEGRO_MOUSE_MAX_EXTRA_AXES   4
 
 
 typedef struct ALLEGRO_MOUSE ALLEGRO_MOUSE;
@@ -53,6 +53,14 @@ struct ALLEGRO_MOUSE_STATE
 };
 
 
+typedef enum ALLEGRO_MOUSE_BUTTON
+{
+   ALLEGRO_MOUSE_BUTTON_LEFT = 1,
+   ALLEGRO_MOUSE_BUTTON_RIGHT = 2,
+   ALLEGRO_MOUSE_BUTTON_MIDDLE = 3,
+} ALLEGRO_MOUSE_BUTTON;
+
+
 AL_FUNC(bool,           al_is_mouse_installed,  (void));
 AL_FUNC(bool,           al_install_mouse,       (void));
 AL_FUNC(void,           al_uninstall_mouse,     (void));
@@ -65,6 +73,7 @@ AL_FUNC(bool,           al_set_mouse_axis,      (int axis, int value));
 AL_FUNC(void,           al_get_mouse_state,     (ALLEGRO_MOUSE_STATE *ret_state));
 AL_FUNC(bool,           al_mouse_button_down,   (const ALLEGRO_MOUSE_STATE *state, int button));
 AL_FUNC(int,            al_get_mouse_state_axis, (const ALLEGRO_MOUSE_STATE *state, int axis));
+AL_FUNC(bool, al_can_get_mouse_cursor_position, (void));
 AL_FUNC(bool, al_get_mouse_cursor_position, (int *ret_x, int *ret_y));
 AL_FUNC(bool, al_grab_mouse, (struct ALLEGRO_DISPLAY *display));
 AL_FUNC(bool, al_ungrab_mouse, (void));
