@@ -1,6 +1,6 @@
-/*         ______   ___    ___ 
- *        /\  _  \ /\_ \  /\_ \ 
- *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
+/*         ______   ___    ___
+ *        /\  _  \ /\_ \  /\_ \
+ *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___
  *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
  *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
  *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
@@ -24,10 +24,7 @@
 
 
 _AL_BEGIN_JOYSTICK_DRIVER_LIST
-#ifdef ALLEGRO_GP2XWIZ
-   _AL_JOYSTICK_DRIVER_GP2XWIZ
-#endif
-#if defined ALLEGRO_HAVE_LINUX_JOYSTICK_H && defined ALLEGRO_WITH_XWINDOWS
+#if defined ALLEGRO_HAVE_LINUX_INPUT_H && (defined ALLEGRO_WITH_XWINDOWS || defined ALLEGRO_RASPBERRYPI)
    { _ALLEGRO_JOYDRV_LINUX,   &_al_joydrv_linux,   true  },
 #endif
 _AL_END_JOYSTICK_DRIVER_LIST
